@@ -24,6 +24,7 @@
 #include <config.h>
 #include "gnome-vfs-inet-connection.h"
 #include "gnome-vfs-private-utils.h"
+#include "gnome-vfs-resolve.h"
 
 #include <errno.h>
 #include <glib/gmem.h>
@@ -40,159 +41,137 @@
 #include <unistd.h>
 #include <arpa/inet.h>
 
-/* AIX #defines h_errno */
-#ifndef h_errno
-extern int h_errno;
-#endif
+#include <sys/time.h>
 
 struct GnomeVFSInetConnection {
-	int family;
-#ifdef ENABLE_IPV6
-	struct sockaddr_in6 addr6;
-#endif
-	struct sockaddr_in addr;
+
+	GnomeVFSAddress *address;	
 	guint sock;
-	guint socklen;
+	struct timeval *timeout;
 };
 
 /**
  * gnome_vfs_inet_connection_create:
- * @connection_return: pointer to a GnomeVFSInetConnection, which will
+ * @connection_return: Pointer to a GnomeVFSInetConnection, which will.
  * contain an allocated GnomeVFSInetConnection object on return.
- * @host_name: string indicating the host to establish an internet connection with
- * @host_port: the port number to connect to
- * @cancellation: handle allowing cancellation of the operation
+ * @host_name: String indicating the host to establish an internet connection with.
+ * @host_port: The port number to connect to.
+ * @cancellation: handle allowing cancellation of the operation.
  *
- * Creates a connection at @handle_return to @host_name using
+ * Creates a connection at @connection_return to @host_name using
  * port @port.
  *
- * Return value: GnomeVFSResult indicating the success of the operation
+ * Return value: #GnomeVFSResult indicating the success of the operation.
  **/
 GnomeVFSResult
 gnome_vfs_inet_connection_create (GnomeVFSInetConnection **connection_return,
-				  const gchar *host_name,
-				  guint host_port,
-				  GnomeVFSCancellation *cancellation)
+				  const gchar             *host_name,
+				  guint                    host_port,
+				  GnomeVFSCancellation    *cancellation)
 {
 	GnomeVFSInetConnection *new;
-#ifdef ENABLE_IPV6
-	struct addrinfo hints, *result, *res;
-	int ret;
-#endif
-	struct hostent *host_info;
-	struct sockaddr_in addr;
-	gint sock;
+	GnomeVFSResolveHandle *rh;
+	GnomeVFSAddress *address;
+	GnomeVFSResult res;
+	gint sock, len, ret;
+	struct sockaddr *saddr;
 
 	g_return_val_if_fail (connection_return != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 	g_return_val_if_fail (host_name != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 	g_return_val_if_fail (host_port != 0, GNOME_VFS_ERROR_BAD_PARAMETERS);
 
-#ifdef ENABLE_IPV6
-	if (_gnome_vfs_have_ipv6 ()) {
-		result = NULL;
-		ret = 0;
-		sock = 0;
+	res = gnome_vfs_resolve (host_name, &rh);
 
-		memset (&hints, 0, sizeof (hints));
-		hints.ai_socktype = SOCK_STREAM;
+	if (res != GNOME_VFS_OK)
+		return res;
 
-		if (getaddrinfo (host_name, NULL, &hints, &result) != 0) {
-			return GNOME_VFS_ERROR_HOST_NOT_FOUND;
-		}
+	sock = -1;
+	
+	while (gnome_vfs_resolve_next_address (rh, &address)) {
+		sock = socket (gnome_vfs_address_get_family_type (address),
+			       SOCK_STREAM, 0);
 
-		if (gnome_vfs_cancellation_check (cancellation)) {
-			return GNOME_VFS_ERROR_CANCELLED;
-		}
-
-		/*For selecting a valid IP of the List*/
-		for (res = result; res; res = res->ai_next) {
-
-			if (res->ai_family != AF_INET && res->ai_family != AF_INET6) {
-				continue;
-			}
-
-			sock = socket (res->ai_family, SOCK_STREAM, 0);
-			if (sock < 0) {
-				continue;
-			}
-
-			if (res->ai_family == AF_INET) {
-				((struct sockaddr_in *)res->ai_addr)->sin_port = htons (host_port);
-			}
-
-			if (res->ai_family == AF_INET6) {
-				((struct sockaddr_in6 *)res->ai_addr)->sin6_port = htons (host_port);
-			}
-
-			ret = connect (sock, res->ai_addr, res->ai_addrlen);
-			if (ret == 0) {
+		if (sock > -1) {
+			saddr = gnome_vfs_address_get_sockaddr (address,
+								host_port,
+								&len);
+			ret = connect (sock, saddr, len);
+			g_free (saddr);
+			
+			if (ret == 0)
 				break;
-			}
 
 			close (sock);
+			sock = -1;
 		}
 
-		if (!res) {
-			freeaddrinfo (result);
-
-			if (sock < 0 || ret < 0) {
-				/*Error in connection or socket creation.*/
-				return gnome_vfs_result_from_errno ();
-			} else {
-				/*Error: No IPv4 or IPv6 address.*/
-				return GNOME_VFS_ERROR_HOST_NOT_FOUND;
-			}
-		}
-
-		new = g_new (GnomeVFSInetConnection, 1);
-
-		if (res->ai_family == AF_INET) {
-			new->family = AF_INET;
-			memcpy (&new->addr, res->ai_addr, res->ai_addrlen);
-		}
-
-		if (res->ai_family == AF_INET6) {
-			new->family = AF_INET6;
-			memcpy (&new->addr6, res->ai_addr, res->ai_addrlen);
-		}
-
-		new->socklen = res->ai_addrlen;
-		new->sock = sock;
-		freeaddrinfo (result);
+		gnome_vfs_address_free (address);
+		
 	}
-	else
-#endif
-	{
-		sock = socket (PF_INET, SOCK_STREAM, 0);
-		if (sock < 0) {
-			return gnome_vfs_result_from_errno ();
-		}
 
-		host_info = gethostbyname (host_name);
-		if (gnome_vfs_cancellation_check (cancellation)) {
-			return GNOME_VFS_ERROR_CANCELLED;
-		}
+	gnome_vfs_resolve_free (rh);
+	
+	if (sock < 0)
+		return gnome_vfs_result_from_errno ();
+	
+	new = g_new0 (GnomeVFSInetConnection, 1);
+	new->address = address;
+	new->sock = sock;
 
-		if (host_info == NULL) {
-			return gnome_vfs_result_from_h_errno ();
-		}
+	_gnome_vfs_set_fd_flags (new->sock, O_NONBLOCK);
 
-		addr.sin_family = host_info->h_addrtype;
-		addr.sin_addr = * (struct in_addr *) host_info->h_addr;
-		addr.sin_port = htons (host_port);
+	*connection_return = new;
+	return GNOME_VFS_OK;
+}
 
-		if (connect (sock, (struct sockaddr *) &addr, sizeof (addr)) < 0) {
-			return gnome_vfs_result_from_errno ();
-		}
+/**
+ * gnome_vfs_inet_connection_create_from_address:
+ * @connection_return: Pointer to a GnomeVFSInetConnection, which will.
+ * contain An allocated GnomeVFSInetConnection object on return.
+ * @address: A valid #GnomeVFSAddress.
+ * @host_port: The port number to connect to.
+ * @cancellation: Handle allowing cancellation of the operation.
+ *
+ * Creates a connection at @connection_return to @address using
+ * port @port.
+ *
+ * Return value: #GnomeVFSResult indicating the success of the operation.
+ *
+ * Since: 2.8
+ **/
+GnomeVFSResult
+gnome_vfs_inet_connection_create_from_address (GnomeVFSInetConnection **connection_return,
+					       GnomeVFSAddress         *address,
+					       guint                    host_port,
+					       GnomeVFSCancellation    *cancellation)
+{
+	GnomeVFSInetConnection *new;
+	gint sock, len, ret;
+	struct sockaddr *saddr;
+	
+	sock = socket (gnome_vfs_address_get_family_type (address),
+		       SOCK_STREAM, 0);
 
-		new = g_new (GnomeVFSInetConnection, 1);
-		new->family = AF_INET;
-		memcpy (&new->addr, &addr, sizeof (addr));
-		new->socklen = sizeof(addr);
-		new->sock = sock;
+	if (sock < 0)
+		return gnome_vfs_result_from_errno ();
+	
+	saddr = gnome_vfs_address_get_sockaddr (address,
+						host_port,
+						&len);
+	
+	ret = connect (sock, saddr, len);
+	g_free (saddr);
 
+	if (ret < 0) {
+		close (sock);
+		return gnome_vfs_result_from_errno ();
 	}
 
+		
+	new = g_new0 (GnomeVFSInetConnection, 1);
+	new->address = gnome_vfs_address_dup (address);
+	new->sock = sock;
+
 	_gnome_vfs_set_fd_flags (new->sock, O_NONBLOCK);
 
 	*connection_return = new;
@@ -202,8 +181,8 @@ gnome_vfs_inet_connection_create (GnomeVFSInetConnection **connection_return,
 
 /**
  * gnome_vfs_inet_connection_destroy:
- * @connection: connection to destroy
- * @cancellation: handle for cancelling the operation
+ * @connection: Connection to destroy.
+ * @cancellation: Handle for cancelling the operation.
  *
  * Closes/Destroys @connection.
  **/
@@ -214,12 +193,38 @@ gnome_vfs_inet_connection_destroy (GnomeVFSInetConnection *connection,
 	g_return_if_fail (connection != NULL);
 
 	close (connection->sock);
+	
+	gnome_vfs_inet_connection_free (connection, cancellation);
+}
+
+
+/**
+ * gnome_vfs_inet_connection_free:
+ * @connection: Connection to free.
+ * @cancellation: Handle for cancelling the operation.
+ *
+ * Frees @connection without closing the socket.
+ **/
+void
+gnome_vfs_inet_connection_free (GnomeVFSInetConnection *connection,
+				GnomeVFSCancellation *cancellation)
+{
+	g_return_if_fail (connection != NULL);
+	
+	if (connection->timeout)
+		g_free (connection->timeout);
+
+	if (connection->address)
+		gnome_vfs_address_free (connection->address);
+
 	g_free (connection);
+	
 }
 
 /**
  * gnome_vfs_inet_connection_close:
  * @connection: connection to close
+ * @cancellation: handle allowing cancellation of the operation
  *
  * Closes @connection, freeing all used resources.
  **/
@@ -232,7 +237,7 @@ gnome_vfs_inet_connection_close (GnomeVFSInetConnection *connection,
 
 /**
  * gnome_vfs_inet_connection_get_fd:
- * @connection: connection to get the file descriptor from
+ * @connection: Connection to get the file descriptor from
  *
  * Retrieve the UNIX file descriptor corresponding to @connection.
  *
@@ -247,53 +252,46 @@ gnome_vfs_inet_connection_get_fd (GnomeVFSInetConnection *connection)
 
 /**
  * gnome_vfs_inet_connection_get_ip:
- * @connection: connection to get the ip from
+ * @connection: Connection to get the ip from.
  *
  * Retrieve the ip address of the other side of a connected @connection.
  *
- * Return value: string version of the ip
+ * Return value: String version of the ip.
  *
  * Since: 2.8
  **/
 char *
 gnome_vfs_inet_connection_get_ip (GnomeVFSInetConnection *connection)
 {
-	const char *res;
-#ifdef ENABLE_IPV6
-	char ip[INET6_ADDRSTRLEN];
-#else
-	char ip[INET_ADDRSTRLEN];
-#endif
-
-	res = NULL;
-#ifdef ENABLE_IPV6
-	if (connection->family == AF_INET6) {
-		res = inet_ntop (AF_INET6, &connection->addr6.sin6_addr,
-				 ip, sizeof(ip));
-	}
-#endif
-	if (connection->family == AF_INET) {
-		res = inet_ntop (AF_INET, &connection->addr.sin_addr,
-				 ip, sizeof(ip));
-	}
-	
-
-	if (res != NULL) {
-		return g_strdup (ip);
-	}
-	return NULL;
+	return gnome_vfs_address_to_string (connection->address);
 }
 
+/**
+ * gnome_vfs_inet_connection_get_address:
+ * @connection: Connection to get the address from. 
+ *
+ * Retrieve the address of the other side of a connected @connection.
+ * 
+ * Return Value: A #GnomeVFSAddress containing the address.
+ *
+ * Since 2.8
+ **/
+GnomeVFSAddress *
+gnome_vfs_inet_connection_get_address (GnomeVFSInetConnection *connection)
+{
+	return gnome_vfs_address_dup (connection->address);
+}
 
 /* SocketImpl for InetConnections */
 
 /**
  * gnome_vfs_inet_connection_read:
- * @connection: connection to read data from
- * @buffer: allocated buffer of at least @bytes bytes to be read into
- * @bytes: number of bytes to read from @socket into @buffer
- * @bytes_read: pointer to a GnomeVFSFileSize, will contain
+ * @connection: Connection to read data from.
+ * @buffer: Allocated buffer of at least @bytes bytes to be read into.
+ * @bytes: Number of bytes to read from @socket into @buffer.
+ * @bytes_read: Pointer to a GnomeVFSFileSize, will contain
  * the number of bytes actually read from the socket on return.
+ * @cancellation: Handle allowing cancellation of the operation.
  *
  * Read @bytes bytes of data from @connection into @buffer.
  *
@@ -309,6 +307,7 @@ gnome_vfs_inet_connection_read (GnomeVFSInetConnection *connection,
 	gint     read_val;
 	fd_set   read_fds;
 	int max_fd, cancel_fd;
+	struct timeval timeout;
 
 	cancel_fd = -1;
 	
@@ -327,10 +326,19 @@ read_loop:
 			max_fd = MAX (max_fd, cancel_fd);
 		}
 		
-		read_val = select (max_fd + 1, &read_fds, NULL, NULL, NULL);
-
-		if (read_val != -1) { 	
-
+		/* select modifies the timeval struct so set it every loop */
+		if (connection->timeout != NULL) {
+			timeout.tv_sec = connection->timeout->tv_sec;
+			timeout.tv_usec = connection->timeout->tv_usec;
+		}
+		
+		read_val = select (max_fd + 1, &read_fds, NULL, NULL,
+				   connection->timeout ? &timeout : NULL);
+		
+		if (read_val == 0) {
+			return GNOME_VFS_ERROR_TIMEOUT;
+		} else if (read_val != -1) { 	
+			
 			if (cancel_fd != -1 && FD_ISSET (cancel_fd, &read_fds)) {
 				return GNOME_VFS_ERROR_CANCELLED;
 			}
@@ -364,11 +372,12 @@ read_loop:
 
 /**
  * gnome_vfs_inet_connection_write:
- * @connection: connection to write data to
- * @buffer: data to write to the connection
- * @bytes: number of bytes from @buffer to write to @socket
- * @bytes_written: pointer to a GnomeVFSFileSize, will contain
+ * @connection: Connection to write data to.
+ * @buffer: Data to write to the connection.
+ * @bytes: Number of bytes from @buffer to write to @socket.
+ * @bytes_written: Pointer to a GnomeVFSFileSize, will contain
  * the number of bytes actually written to the connection on return.
+ * @cancellation: Handle allowing cancellation of the operation.
  *
  * Write @bytes bytes of data from @buffer to @connection.
  *
@@ -382,11 +391,13 @@ gnome_vfs_inet_connection_write (GnomeVFSInetConnection *connection,
 				 GnomeVFSCancellation *cancellation)
 {
 	gint    write_val;
-	fd_set  write_fds;
+	fd_set  write_fds, *read_fds, pipe_fds;
 	int max_fd, cancel_fd;
+	struct timeval timeout;
 
 
 	cancel_fd = -1;
+	read_fds  = NULL;
 	
 write_loop:	
 	write_val = write (connection->sock, buffer, bytes);
@@ -399,15 +410,27 @@ write_loop:
 
 		if (cancellation != NULL) {
 			cancel_fd = gnome_vfs_cancellation_get_fd (cancellation);
-			FD_SET (cancel_fd, &write_fds);
+			read_fds = &pipe_fds;
+			FD_ZERO (read_fds);
+			FD_SET (cancel_fd, read_fds);
 			max_fd = MAX (max_fd, cancel_fd);
 		}
-		
-		write_val = select (max_fd + 1, NULL, &write_fds, NULL, NULL);
-				
-		if (write_val != -1) { 	
 
-			if (cancel_fd != -1 && FD_ISSET (cancel_fd, &write_fds)) {
+		/* select modifies the timeval struct so set it every loop */
+		if (connection->timeout != NULL) {
+			timeout.tv_sec = connection->timeout->tv_sec;
+			timeout.tv_usec = connection->timeout->tv_usec;
+		}
+
+		
+		write_val = select (max_fd + 1, read_fds, &write_fds, NULL,
+					connection->timeout ? &timeout : NULL);
+				
+		if (write_val == 0) {
+			return GNOME_VFS_ERROR_TIMEOUT;
+		} else if (write_val != -1) {
+
+			if (cancel_fd != -1 && FD_ISSET (cancel_fd, read_fds)) {
 				return GNOME_VFS_ERROR_CANCELLED;
 			}
 			
@@ -437,15 +460,56 @@ write_loop:
 	}
 }
 
+/**
+ * gnome_vfs_inet_connection_set_timeout:
+ * @connection: Connection to set the timeout of.
+ * @timeout: The timeout to set.
+ * @cancellation: Optional cancellation object.
+ *
+ * Set a timeout of @timeout. If @timeout is NULL following operations
+ * will block indefinitely).
+ *
+ * Note if you set @timeout to 0 (means tv_sec and tv_usec are both 0)
+ * every following operation will return immediately. (This can be used
+ * for polling.)
+ *
+ * Return value: GnomeVFSResult indicating the success of the operation
+ *
+ * Since: 2.8
+ **/
+
+
+static GnomeVFSResult
+gnome_vfs_inet_connection_set_timeout (GnomeVFSInetConnection *connection,
+				       GTimeVal *timeout,
+				       GnomeVFSCancellation *cancellation)
+{
+	if (timeout == NULL) {
+		if (connection->timeout != NULL) {
+			g_free (connection->timeout);
+			connection->timeout = NULL;
+		}
+	} else {
+		if (connection->timeout == NULL)
+			connection->timeout = g_new0 (struct timeval, 1);
+
+		connection->timeout->tv_sec  = timeout->tv_sec;
+		connection->timeout->tv_usec = timeout->tv_usec;
+	}
+	
+	return GNOME_VFS_OK;
+}
+
 static GnomeVFSSocketImpl inet_connection_socket_impl = {
 	(GnomeVFSSocketReadFunc)gnome_vfs_inet_connection_read,
 	(GnomeVFSSocketWriteFunc)gnome_vfs_inet_connection_write,
-	(GnomeVFSSocketCloseFunc)gnome_vfs_inet_connection_close
+	(GnomeVFSSocketCloseFunc)gnome_vfs_inet_connection_close,
+	(GnomeVFSSocketSetTimeoutFunc)gnome_vfs_inet_connection_set_timeout
 };
 
 /**
  * gnome_vfs_inet_connection_to_socket:
- * @connection: connection to convert to wrapper in a GnomeVFSSocket
+ * @connection: Connection to convert to wrapper in a GnomeVFSSocket.
  *
  * Wrapper @connection inside a standard GnomeVFSSocket for convenience.
  *
@@ -459,7 +523,7 @@ gnome_vfs_inet_connection_to_socket (GnomeVFSInetConnection *connection)
 
 /**
  * gnome_vfs_inet_connection_to_socket_buffer:
- * @connection: connection to convert to wrapper in a GnomeVFSSocketBuffer
+ * @connection: Connection to convert to wrapper in a GnomeVFSSocketBuffer.
  *
  * Wrapper @connection inside a standard GnomeVFSSocketBuffer for convenience.
  *
