@@ -384,7 +384,9 @@ set_uri_element (GnomeVFSURI *uri,
 	 * meaning.
 	 */
 	if ( ! (strcmp (uri->method_string, "http") == 0 
-	        || strcmp (uri->method_string, "eazel-services") == 0
+	        || strcmp (uri->method_string, "https") == 0
+		|| strcmp (uri->method_string, "dav") == 0
+		|| strcmp (uri->method_string, "davs") == 0
 	        || strcmp (uri->method_string, "ghelp") == 0
 	        || strcmp (uri->method_string, "gnome-help") == 0
 	        || strcmp (uri->method_string, "help") == 0
@@ -1742,18 +1744,21 @@ gnome_vfs_uri_extract_short_name (const GnomeVFSURI *uri)
 	short_path_name = gnome_vfs_unescape_string (escaped_short_path_name, "/");
 	g_free (escaped_short_path_name);
 
-	host_name = NULL;
 	if (short_path_name != NULL
-		&& strcmp (short_path_name, GNOME_VFS_URI_PATH_STR) == 0) {
-		host_name = gnome_vfs_uri_get_host_name (uri);
-	}
-
-	if (host_name == NULL || strlen (host_name) == 0) {
+		&& strcmp (short_path_name, GNOME_VFS_URI_PATH_STR) != 0) {
 		return short_path_name;
 	}
 
-	g_free (short_path_name);
-	return g_strdup (host_name);
+	host_name = gnome_vfs_uri_get_host_name (uri);
+
+	if (host_name != NULL && strlen (host_name) != 0) {
+		g_free (short_path_name);
+		return g_strdup (host_name);
+	} else if (short_path_name != NULL) {
+		return short_path_name;
+	} else {
+		return g_strdup (gnome_vfs_uri_get_path (uri));
+	}
 }
 
 /**
