@@ -71,6 +71,7 @@
 #define DEFAULT_HTTP_PORT 	80
 #define DEFAULT_HTTPS_PORT 	443
 
+#define HTTP_DIR_MIME_TYPE      "x-directory/webdav"
 
 /* ************************************************************************** */
 /* module interfaces */
@@ -96,8 +97,8 @@ void http_debug_printf(const char *func, const char *fmt, ...) G_GNUC_PRINTF (2,
 
 
 #define DEBUG_HTTP_CONTEXT(c) http_debug_printf (__PRETTY_FUNCTION__,       \
-						  "[Context] Session :%p, path: %s", \
-						  c->session, c->path)
+						  "[Context] Session :%p, path: %s, dav_mode %d", \
+						  c->session, c->path, c->dav_mode)
 
 void
 http_debug_printf (const char *func, const char *fmt, ...)
@@ -274,22 +275,31 @@ resolve_result (int result, ne_request *request)
 /* ************************************************************************** */
 /* Utility functions */
 
+typedef enum {
+	
+	DAV_CLASS_NOT_SET = -1,
+	NO_DAV            =  0,
+	DAV_CLASS_1       =  1,
+	DAV_CLASS_2       =  2
+
+} DavClass;
+
 typedef struct {
 	
 	char      *scheme;
 	gboolean   ssl;
 	char      *alias;
+	gboolean   is_dav;
 	   
 } MethodSchemes;
 
-
 MethodSchemes supported_schemes[] = {
 
-	{"http",  FALSE, "http" },
-	{"dav",   FALSE, "http" },
-	{"https", TRUE,  "https"},
-	{"davs",  TRUE,  "https"},
-	{NULL,    FALSE         }	   
+	{"http",  FALSE, "http",  FALSE},
+	{"dav",   FALSE, "http",  TRUE},
+	{"https", TRUE,  "https", FALSE},
+	{"davs",  TRUE,  "https", TRUE},
+	{NULL,    FALSE,          FALSE}	   
 };
 
 static char *
@@ -311,6 +321,22 @@ resolve_alias (const char *scheme)
 	return iter->alias;
 }
 
+static gboolean
+scheme_is_dav (GnomeVFSURI *uri)
+{
+	const char *scheme;
+
+	scheme = gnome_vfs_uri_get_scheme (uri);
+	
+	if (scheme == NULL)
+		return FALSE;
+
+	if (! g_ascii_strcasecmp (scheme, "dav") ||
+	    ! g_ascii_strcasecmp (scheme, "davs"))
+		return TRUE;
+
+	return FALSE;
+}
 
 static guint
 http_session_uri_hash (gconstpointer v)
@@ -963,10 +989,10 @@ static void
 set_dav_class (void *userdata, const char *value)
 {
 	char *tokens = ne_strdup(value), *pnt = tokens;
-	gint *dav_class = userdata;
+        DavClass *dav_class = userdata;
     
 	DEBUG_HTTP_3 ("parsing dav: %s", tokens); 
-	
+
 	do {
 		char *tok = ne_qtoken (&pnt, ',',  "\"'");
 		if (!tok) break;
@@ -1087,7 +1113,8 @@ set_content_range (void *ud, const char *value)
     	}
 }
 
-
+/* function for setting the etag value
+   we dont use it at the moment
 static void 
 set_etag (void **etag, const char *value)
 {		
@@ -1096,6 +1123,30 @@ set_etag (void **etag, const char *value)
 	else
 		*etag = NULL;
 }
+*/
+
+static void
+add_default_header_handlers (ne_request *req, GnomeVFSFileInfo *info)
+{
+
+	ne_add_response_header_handler (req, "Last-Modified", 
+					(ne_header_handler) set_last_modified,
+		   			info);
+	
+	ne_add_response_header_handler (req, "Content-Length", 
+					(ne_header_handler) set_content_length,
+					info);
+	
+	ne_add_response_header_handler (req, "Content-Type",
+					(ne_header_handler) set_content_type,
+					info);
+	
+	ne_add_response_header_handler (req, "Date",
+					(ne_header_handler) set_access_time,
+					info);
+       
+}
+
 
 /* ************************************************************************** */
 /* Propfind request handlers */
@@ -1105,7 +1156,7 @@ static const ne_propname file_info_props[] = {
 	{"DAV:", "resourcetype"},
 	{"DAV:", "getcontenttype"},
 	{"DAV:", "getcontentlength"},
-	{"DAV:", "getetag"},
+	/*      {"DAV:", "getetag"}, */
 	/*	{"http://apache.org/dav/props/", "executable"}, */
 	{NULL}
 };
@@ -1137,15 +1188,23 @@ propfind_context_init (PropfindContext *pfctx)
 static void
 propfind_context_clear (PropfindContext *pfctx)
 {
-	if (pfctx->target != NULL)
-		gnome_vfs_file_info_unref (pfctx->target);
+	GList *l;
 	
+	if (pfctx->target != NULL) {
+		gnome_vfs_file_info_unref (pfctx->target);
+		pfctx->target = NULL;
+	}
+		
 	if (pfctx->children) {
-		g_list_free (pfctx->children);
+		l = gnome_vfs_file_info_list_unref (pfctx->children);
+		g_list_free (l);
+		pfctx->children = NULL;
 	}
 	
-	if (pfctx->etag)
+	if (pfctx->etag) {
 		g_free (pfctx->etag);
+		pfctx->etag = NULL;
+	}
 }
 
 
@@ -1177,16 +1236,20 @@ propfind_result (void *userdata, const char *href, const ne_prop_result_set *set
 	info->name = g_path_get_basename (unesc_path);
 	NE_FREE (unesc_path);
 	
+	DEBUG_HTTP_2 ("Comparing: \n\t[%s] \n\t[%s]", ctx->path, uri.path);
 	if (ne_path_compare (ctx->path, uri.path) == 0) {
 		DEBUG_HTTP_3 ("target");
 		ctx->target = info;
 		
 		/* Set the etag on target */
+		/* We are not requesting the etag information at 
+		 * the moment so don't even check for it 
 		value = ne_propset_value (set, &file_info_props[5]);
 		
 		if (value != NULL)
 			ctx->etag = g_strdup (value);
-		
+	
+		*/	
 	} else {
 		ctx->children = g_list_append (ctx->children, info);
 	}
@@ -1214,7 +1277,7 @@ propfind_result (void *userdata, const char *href, const ne_prop_result_set *set
 	info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_TYPE;
 	
 	if (value != NULL && strstr (value, "collection")) {
-		info->mime_type = g_strdup ("x-directory/webdav");
+		info->mime_type = g_strdup (HTTP_DIR_MIME_TYPE);
 		info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE;
 		info->type = GNOME_VFS_FILE_TYPE_DIRECTORY;
 		
@@ -1515,11 +1578,12 @@ typedef struct {
 	const char *scheme;
 	gboolean ssl;
 	
-	guint dav_class;
+	DavClass dav_class;
 	HttpMethods methods;
 	
 	ne_session *session;
-	
+
+	gboolean dav_mode;
 	gboolean redirected;
 	guint    redir_count;
 	
@@ -1661,7 +1725,9 @@ http_context_set_uri (HttpContext *context, GnomeVFSURI *uri)
 		uri_string = g_strdup ("/");
 	}
 	
-	context->path = uri_string;
+	context->path      = uri_string;
+	context->methods   = 0;
+	context->dav_class = DAV_CLASS_NOT_SET;
 	
 }
 
@@ -1678,7 +1744,7 @@ http_context_open (GnomeVFSURI *uri, HttpContext **context)
 	ctx = g_new0 (HttpContext, 1);
 	
 	http_context_set_uri (ctx, uri);
-
+	
 	if (ctx->scheme == NULL) {
 		http_context_free (ctx);
 		return GNOME_VFS_ERROR_NOT_SUPPORTED;
@@ -1692,8 +1758,10 @@ http_context_open (GnomeVFSURI *uri, HttpContext **context)
 		return result;
 	}
 
-	*context = ctx;
+	ctx->dav_mode = scheme_is_dav (uri);
 	
+	*context = ctx;
+
 	return GNOME_VFS_OK;
 }
 
@@ -1743,22 +1811,20 @@ http_follow_redirect (HttpContext *context)
 /* Http operations */
 
 static GnomeVFSResult
-http_get_file_info (HttpContext *context, GnomeVFSFileInfo *info, char **etag)
+http_get_file_info (HttpContext *context, GnomeVFSFileInfo *info)
 {
 	GnomeVFSResult result;
 	PropfindContext pfctx;
 	ne_propfind_handler *pfh;
-	char *_etag;
 	ne_request *req;
 	int res;
 
 	DEBUG_HTTP_CONTEXT (context);
 	
 	/* no dav server */
-	if (context->dav_class == -1) 
+	if (context->dav_mode == FALSE || context->dav_class == NO_DAV) 
 		goto head_start;
  	
-	_etag = NULL;
 	propfind_context_init (&pfctx);
 	
  propfind_start:	
@@ -1784,19 +1850,18 @@ http_get_file_info (HttpContext *context, GnomeVFSFileInfo *info, char **etag)
 	
 	/* Let's be very cautious here! If the server doesn't respond with a 
 	   207 here just fall back to HEAD because some server server (eg. gws) 
-	   colse the connection on an unknown command, most (stupid) php scripts
+	   close the connection on an unknown command, most (stupid) php scripts
 	   will treat PROPFIND as GET and some servers may deny us PROFIND but
 	   allow us HEAD. */
 	if (res == NE_OK && ne_get_status (req)->code == 207) {
 		
-		if (result == GNOME_VFS_OK) {
-			gnome_vfs_file_info_copy (info, pfctx.target);
-			
-			if (etag && pfctx.etag)
-				*etag = pfctx.etag;
-			pfctx.etag = NULL;
+
+		if (pfctx.target != NULL) {
+			gnome_vfs_file_info_copy (info, pfctx.target);	
+		} else {
+			result = GNOME_VFS_ERROR_NOT_FOUND;
 		}
-			
+		
 		propfind_context_clear (&pfctx);
 		return result;
 	}
@@ -1806,27 +1871,8 @@ http_get_file_info (HttpContext *context, GnomeVFSFileInfo *info, char **etag)
 	
  head_start:	
 	req  = ne_request_create (context->session, "HEAD", context->path);
-			
-	ne_add_response_header_handler (req, "Last-Modified", 
-					(ne_header_handler) set_last_modified,
-		   			info);
-	
-	ne_add_response_header_handler (req, "Content-Length", 
-					(ne_header_handler) set_content_length,
-					info);
-	
-	ne_add_response_header_handler (req, "Content-Type",
-					(ne_header_handler) set_content_type,
-					info);
-	
-	ne_add_response_header_handler (req, "Date",
-					(ne_header_handler) set_access_time,
-					info);
 
-	ne_add_response_header_handler (req, "ETag",
-					(ne_header_handler) set_etag,
-					&_etag);
-			
+	add_default_header_handlers (req, info);
 		
 	res = ne_request_dispatch (req);
 
@@ -1851,19 +1897,28 @@ http_get_file_info (HttpContext *context, GnomeVFSFileInfo *info, char **etag)
 		info->flags = GNOME_VFS_FILE_FLAGS_NONE;
 		
 		info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_TYPE;
-		
-		if (etag && _etag) {
-			*etag = _etag;
-			_etag = NULL;
+
+		/* work-around for broken icecast server */
+		if (! g_ascii_strcasecmp (info->mime_type, "audio/mpeg")) {
+			ne_close_connection (ne_get_session (req));
 		}
+		
 	}
 
-	if (_etag)
-		g_free (_etag);
-	
 	return result;
 }
 
+static void assure_trailing_slash (HttpContext *context)
+{
+	char *tofree;
+
+	if (! ne_path_has_trailing_slash (context->path)) {
+		tofree = context->path;
+		context->path = g_strconcat (tofree, "/", NULL);
+		g_free (tofree);
+	}
+}
+
 static GnomeVFSResult
 http_list_directory (HttpContext *context, PropfindContext *pfctx)
 {
@@ -1871,31 +1926,18 @@ http_list_directory (HttpContext *context, PropfindContext *pfctx)
 	ne_propfind_handler *pfh;
 	ne_request *req;
 	int res;
-	char *tofree;
 
 	DEBUG_HTTP_CONTEXT (context);
 	
-	tofree = NULL;
 	propfind_context_init (pfctx);
+	pfctx->path = context->path;
 	
  propfind_start:	
-	if (ne_path_has_trailing_slash (context->path)) {
-		pfctx->path = context->path;
-	} else {
-		tofree = g_strconcat (context->path, "/", NULL);
-		pfctx->path = tofree;
-	}
-	
 	pfctx->include_target = TRUE;
-
+	
 	pfh = ne_propfind_create (context->session, context->path, 1);
 	res = ne_propfind_named (pfh, file_info_props, propfind_result, pfctx);
 	
-	if (tofree != NULL) {
-		g_free (tofree);
-		tofree = NULL;
-	}
-
 	if (res == NE_REDIRECT) {
 		
 		ne_propfind_destroy (pfh);
@@ -1913,22 +1955,26 @@ http_list_directory (HttpContext *context, PropfindContext *pfctx)
 	result 	= resolve_result (res, req);
 	
 	ne_propfind_destroy (pfh);
+
+	if (result == GNOME_VFS_OK && pfctx->target == NULL) {
+		return GNOME_VFS_ERROR_NOT_FOUND;
+	}
 	
 	return result;
 }
 
 
 static GnomeVFSResult
-http_options (HttpContext *hctx, gint *dav_class)
+http_options (HttpContext *hctx)
 {
 	GnomeVFSResult  result;
 	ne_request     *req;
-	gint 		klass;
+	DavClass 	klass;
 	HttpMethods	methods;
 	int		res;
 	
  options_start:	
-	klass = -1;
+	klass = DAV_CLASS_NOT_SET;
 	methods = 0;
 	req = ne_request_create (hctx->session, "OPTIONS", hctx->path);
 	ne_add_response_header_handler (req, "DAV", set_dav_class, &klass);
@@ -1945,9 +1991,7 @@ http_options (HttpContext *hctx, gint *dav_class)
 	result = resolve_result (res, req);
 	
 	if (result == GNOME_VFS_OK) {
-		if (dav_class)
-			*dav_class = klass;
-	
+		
 		hctx->dav_class = klass;
 		hctx->methods = methods;
 	}
@@ -2013,11 +2057,11 @@ dav_request (ne_request *req, gboolean allow_redirect)
 
 typedef struct {
 	
-	HttpContext      *context;
-	GnomeVFSOpenMode  mode;
-	GnomeVFSFileInfo  *info;
-	GnomeVFSFileOffset offset;
-	char *etag;
+	HttpContext        *context;
+	GnomeVFSOpenMode    mode;
+	GnomeVFSFileInfo   *info;
+	GnomeVFSFileOffset  offset;
+	char               *etag;
 	
 	/* Whether we do/can ranged gets or not */
 	gboolean can_range;
@@ -2118,7 +2162,7 @@ http_transfer_start_write (HttpFileHandle *handle)
 	
 	/* Check to see if we have any file information. If not fetch it!*/
 	if (handle->info->valid_fields == 0) {
-		result = http_get_file_info (hctx, handle->info, NULL);
+		result = http_get_file_info (hctx, handle->info);
 
 		if (result != GNOME_VFS_OK) {
 			return result;
@@ -2176,27 +2220,8 @@ get_start:
 						&range);
 	}
 	
-		
-	ne_add_response_header_handler (req, "Last-Modified", 
-					(ne_header_handler) set_last_modified,
-		   			handle->info);
-	
-	ne_add_response_header_handler (req, "Content-Length", 
-					(ne_header_handler) set_content_length,
-					handle->info);
-	
-	ne_add_response_header_handler (req, "Content-Type",
-					(ne_header_handler) set_content_type,
-					handle->info);
-	
-	ne_add_response_header_handler (req, "Date",
-					(ne_header_handler) set_access_time,
-					handle->info);
-	
-	ne_add_response_header_handler (req, "ETag",
-					(ne_header_handler) set_etag,
-					&(handle->etag));
-	
+
+	add_default_header_handlers (req, handle->info);
 	
 	res = ne_begin_request (req);
 	result = resolve_result (res, req);
@@ -2309,7 +2334,7 @@ do_open (GnomeVFSMethod 	*method,
 	hctx = handle->context;
 
 	if (mode & GNOME_VFS_OPEN_WRITE) {
-		if ((result = http_options (hctx, NULL)) != GNOME_VFS_OK) {
+		if ((result = http_options (hctx)) != GNOME_VFS_OK) {
 			http_file_handle_destroy (handle);
 			return result;
 		}
@@ -2392,7 +2417,7 @@ do_create (GnomeVFSMethod	 *method,
 #endif
 	
 	if (exclusive == TRUE) {
-		result = http_get_file_info (hctx, handle->info, NULL);
+		result = http_get_file_info (hctx, handle->info);
 		
 		if (result != GNOME_VFS_ERROR_NOT_FOUND) {
 			http_file_handle_destroy (handle);
@@ -2715,15 +2740,20 @@ do_open_directory (GnomeVFSMethod 	    *method,
 	PropfindContext *pfctx;
 
 	DEBUG_HTTP_FUNC (1);
+
+	if (scheme_is_dav (uri) == FALSE) {
+		return GNOME_VFS_ERROR_NOT_SUPPORTED;
+	}
 	
 	result = http_context_open  (uri, &hctx);
 	
 	if (result != GNOME_VFS_OK)
 		return result;
-	
-	result = http_options (hctx, NULL);
 
-	if (result != GNOME_VFS_OK || hctx->dav_class == -1) {
+	assure_trailing_slash (hctx);
+	result = http_options (hctx);
+
+	if (result != GNOME_VFS_OK || hctx->dav_class == NO_DAV) {
 		http_context_free (hctx);
 		
 		if (result != GNOME_VFS_OK)
@@ -2817,7 +2847,7 @@ do_get_file_info (GnomeVFSMethod 		*method,
 	if (result != GNOME_VFS_OK)
 		return result;
 		
-	result = http_get_file_info (hctx, file_info, NULL);
+	result = http_get_file_info (hctx, file_info);
 	
 	http_context_free (hctx);
 	
@@ -2847,7 +2877,7 @@ do_get_file_info_from_handle (GnomeVFSMethod 		*method,
 		return GNOME_VFS_OK;
 	}
 	
-	result = http_get_file_info (handle->context, handle->info, NULL);
+	result = http_get_file_info (handle->context, handle->info);
 
 	if (result == GNOME_VFS_OK) {
 		gnome_vfs_file_info_copy (file_info, handle->info);
@@ -2880,7 +2910,11 @@ do_make_directory (GnomeVFSMethod  *method,
 	int 		  res;
 	
 	DEBUG_HTTP_FUNC (1);
-	
+
+	if (scheme_is_dav (uri) == FALSE) {
+		return GNOME_VFS_ERROR_NOT_SUPPORTED;
+	}
+		
 	uri_parent = gnome_vfs_uri_get_parent (uri);
 	
 	result = http_context_open (uri_parent, &hctx);
@@ -2888,9 +2922,9 @@ do_make_directory (GnomeVFSMethod  *method,
 	if (result != GNOME_VFS_OK)
 		return result;
 	
-	result = http_options (hctx, NULL);
+	result = http_options (hctx);
 	
-	if (result != GNOME_VFS_OK || (hctx->dav_class == -1)) {
+	if (result != GNOME_VFS_OK || (hctx->dav_class == NO_DAV)) {
 
 		if (result == GNOME_VFS_OK)
 			result = GNOME_VFS_ERROR_NOT_SUPPORTED;
@@ -2940,10 +2974,17 @@ do_remove_directory (GnomeVFSMethod  *method,
 	int  		res;
 	
 	DEBUG_HTTP_FUNC (1);
+
+	if (scheme_is_dav (uri) == FALSE) {
+		return GNOME_VFS_ERROR_NOT_SUPPORTED;
+	}
+	
 	result = http_context_open  (uri, &hctx);
 	
 	if (result != GNOME_VFS_OK)
 		return result;
+
+	assure_trailing_slash (hctx);
 	
 	propfind_context_init (&pfctx);
 	result = http_list_directory (hctx, &pfctx);
@@ -3002,6 +3043,10 @@ do_move (GnomeVFSMethod  *method,
 	
 	DEBUG_HTTP_FUNC (1);
 
+	if (scheme_is_dav (old_uri) == FALSE) {
+		return GNOME_VFS_ERROR_NOT_SUPPORTED;
+	}
+	
 	if (! http_session_uri_equal (old_uri, new_uri)) {
 		return GNOME_VFS_ERROR_NOT_SAME_FILE_SYSTEM;
 	}	
@@ -3059,7 +3104,7 @@ do_unlink (GnomeVFSMethod  *method,
 		return result;
 	
 	file_info = gnome_vfs_file_info_new ();
-	result = http_get_file_info (hctx, file_info, NULL);
+	result = http_get_file_info (hctx, file_info);
 	
 	if (result != GNOME_VFS_OK) {
 		goto out;
