@@ -409,16 +409,35 @@ test_uri_is_parent_shallow (const char *parent, const char *item, gboolean expec
 	test_uri_is_parent_common (parent, item, FALSE, expected_result);
 }
 
+static int
+strcmp_allow_nulls (const char *s1, const char *s2)
+{
+        const char *t1, *t2;
+
+        t1 = (s1 == NULL ? "" : s1);
+        t2 = (s2 == NULL ? "" : s2);
+
+	return strcmp (t1, t2);
+}
+
 #define VERIFY_STRING_RESULT(function, expected) \
 	G_STMT_START {											\
 		char *result = function; 								\
 		if (!((result == NULL && expected == NULL)						\
-		      || (result != NULL && expected != NULL && strcmp (result, (char *)expected) == 0))) {	\
+		      || (strcmp_allow_nulls (result, (char *)expected) == 0))) {			\
 			test_failed ("%s: returned '%s' expected '%s'", #function, result, expected);	\
 		}											\
                 g_free (result);                                                                      \
 	} G_STMT_END
 
+#define VERIFY_STRING_RESULT_NULL(function) \
+	G_STMT_START {											\
+		char *result = function; 								\
+		if (result != NULL) {									\
+			test_failed ("%s: returned '%s' expected '%s'", #function, result, NULL);	\
+		}											\
+	} G_STMT_END
+
 int
 main (int argc, char **argv)
 {
@@ -482,7 +501,7 @@ main (int argc, char **argv)
 	test_uri_to_string ("http://yakk:womble@www.eazel.com:42/blah/", "http://yakk:womble@www.eazel.com:42/blah/", GNOME_VFS_URI_HIDE_NONE);
 
 	test_uri_to_string ("http://yakk:womble@www.eazel.com:42/blah/", "http://:womble@www.eazel.com:42/blah/", GNOME_VFS_URI_HIDE_USER_NAME);
-	test_uri_to_string ("FILE://", "file:", GNOME_VFS_URI_HIDE_NONE);
+	test_uri_to_string ("FILE://", "file:///", GNOME_VFS_URI_HIDE_NONE);
 
 	test_uri_to_string ("file:///trash", "file:///trash", GNOME_VFS_URI_HIDE_NONE);
 	test_uri_to_string ("file:///Users/mikef", "file:///Users/mikef", GNOME_VFS_URI_HIDE_NONE);
@@ -603,15 +622,13 @@ main (int argc, char **argv)
 	test_make_canonical ("http:///&", "http:///&");
 	test_make_canonical ("http:///x", "http:///x");
 	test_make_canonical ("pipe:foo", "pipe:foo");
-	test_make_canonical ("eazel-services:///%3F", "eazel-services:///%3F");
-	test_make_canonical ("eazel-services:///%78", "eazel-services:///x");
-	test_make_canonical ("eazel-services:///?", "eazel-services:///?");
-	test_make_canonical ("eazel-services:///&", "eazel-services:///&");
-	test_make_canonical ("eazel-services:///x", "eazel-services:///x");
-
-	test_make_canonical ("eazel-install://anonymous@/product_name=gnucash", "eazel-install://anonymous@/product_name%3Dgnucash");
-	test_make_canonical ("eazel-install://:password@/product_name=gnucash", "eazel-install://:password@/product_name%3Dgnucash");
+	test_make_canonical ("dav:///%3F", "dav:///%3F");
+	test_make_canonical ("dav:///%78", "dav:///x");
+	test_make_canonical ("dav:///?", "dav:///?");
+	test_make_canonical ("dav:///&", "dav:///&");
+	test_make_canonical ("dav:///x", "dav:///x");
 
+	
 	test_make_canonical ("http://www.eazel.com/query?email=email@eazel.com", "http://www.eazel.com/query?email=email@eazel.com");
 
 	/* test proper case-sensitivity handling */
@@ -637,8 +654,8 @@ main (int argc, char **argv)
 	/* Test chained uris */
 	test_uri_to_string ("/tmp/t.efs#http:///foobar/", "file:///tmp/t.efs#http:/foobar/", GNOME_VFS_URI_HIDE_NONE);
 	test_uri_parent ("/tmp/t.efs#http:/", "file:///tmp/t.efs");
-	test_uri_to_string ("/tmp/t.efs#zip:/", "file:///tmp/t.efs#zip:/", GNOME_VFS_URI_HIDE_NONE);
-	test_uri_parent ("/tmp/t.efs#zip:/", "file:///tmp/t.efs");
+	test_uri_to_string ("/tmp/t.efs#gzip:/", "file:///tmp/t.efs#gzip:/", GNOME_VFS_URI_HIDE_NONE);
+	test_uri_parent ("/tmp/t.efs#gzip:/", "file:///tmp/t.efs");
 	test_uri_to_string ("/tmp/t.efs#unknownmethod:/", "file:///tmp/t.efs", GNOME_VFS_URI_HIDE_NONE);
 
 	/* Test fragment identifiers. */
@@ -699,18 +716,18 @@ main (int argc, char **argv)
 	test_uri_parent ("http://www.eazel.com?///xxx", "http://www.eazel.com?/");
 	test_uri_parent ("http://www.eazel.com/dir?xxx/yyy", "http://www.eazel.com/dir?xxx");
 
-	VERIFY_STRING_RESULT (gnome_vfs_get_local_path_from_uri (""), NULL);
-	VERIFY_STRING_RESULT (gnome_vfs_get_local_path_from_uri ("/#"), NULL);
+	VERIFY_STRING_RESULT_NULL (gnome_vfs_get_local_path_from_uri (""));
+	VERIFY_STRING_RESULT_NULL (gnome_vfs_get_local_path_from_uri ("/#"));
 	VERIFY_STRING_RESULT (gnome_vfs_get_local_path_from_uri ("file:/path"), "/path");
-	VERIFY_STRING_RESULT (gnome_vfs_get_local_path_from_uri ("file://path"), NULL);
+	VERIFY_STRING_RESULT_NULL (gnome_vfs_get_local_path_from_uri ("file://path"));
 	VERIFY_STRING_RESULT (gnome_vfs_get_local_path_from_uri ("file:///path"), "/path");
 	VERIFY_STRING_RESULT (gnome_vfs_get_local_path_from_uri ("file:////path"), "//path");
 	VERIFY_STRING_RESULT (gnome_vfs_get_local_path_from_uri ("file:///my/document.html"), "/my/document.html");
-	VERIFY_STRING_RESULT (gnome_vfs_get_local_path_from_uri ("file:///my/document.html#fragment"), NULL);
+	VERIFY_STRING_RESULT_NULL (gnome_vfs_get_local_path_from_uri ("file:///my/document.html#fragment"));
 	VERIFY_STRING_RESULT (gnome_vfs_get_local_path_from_uri ("file:///my/docu%20ment%23/path"), "/my/docu ment#/path");
-	VERIFY_STRING_RESULT (gnome_vfs_get_local_path_from_uri ("file:///my/docu%20ment%23/path/foo.html.gz#gunzip:///#fragment"), NULL);
-	VERIFY_STRING_RESULT (gnome_vfs_get_local_path_from_uri ("/my/document.html"), NULL);
-	VERIFY_STRING_RESULT (gnome_vfs_get_local_path_from_uri ("http://my/document.html"), NULL);
+	VERIFY_STRING_RESULT_NULL (gnome_vfs_get_local_path_from_uri ("file:///my/docu%20ment%23/path/foo.html.gz#gunzip:///#fragment"));
+	VERIFY_STRING_RESULT_NULL (gnome_vfs_get_local_path_from_uri ("/my/document.html"));
+	VERIFY_STRING_RESULT_NULL (gnome_vfs_get_local_path_from_uri ("http://my/document.html"));
 	
 	/* Testing gnome_vfs_uri_make_full_from_relative */
 	/* (not an extensive testing, but a regression test */
